@@ -1,19 +1,23 @@
 # -*- coding: utf-8 -*-
-#oaepub modules
+
+#Standard Library modules
+from collections import namedtuple
+from keyword import iskeyword
+import logging
+import os
+import shutil
+import sys
+
+#Non-Standard Library modules
+from lxml import etree
+
+#OpenAccess_EPUB modules
 from openaccess_epub import JPTS10_PATH, JPTS11_PATH, JPTS20_PATH,\
     JPTS21_PATH, JPTS22_PATH, JPTS23_PATH, JPTS30_PATH
 from openaccess_epub.utils import element_methods
-#Standard lib modules
-import os
-import sys
-import shutil
-import logging
-from collections import namedtuple
-from keyword import iskeyword
-#Other, nonstandard lib modules
-from lxml import etree
 
-log = logging.getLogger('Article')
+
+log = logging.getLogger('openaccess_epub.article')
 
 dtd_tuple = namedtuple('DTD_Tuple', 'path, name, version')
 
@@ -76,7 +80,7 @@ its DTD.'.format(xml_file))
             self.back = self.document.find('back')  # Element or None
             self.sub_article = self.document.findall('sub-article')  # 0 or more
             self.response = self.document.findall('response')  # 0 or more
-        
+
         #At this point we have parsed the article, validated it, defined key
         #top-level elements in it, and now we must translate its metadata into
         #a data structure.
@@ -88,18 +92,18 @@ its DTD.'.format(xml_file))
         self.publisher = self.get_publisher()
 
     def get_metadata(self):
-        
+
         #Dictionary comprehension - element name : element definition
         dtd_dict = {i.name: i for i in self.dtd.elements()}
-        
+
         def coerce_string(input):
             for char in input:
                 if char.lower() not in 'abcdefghijklmnopqrstuvwxyz1234567890_':
                     input = input.replace(char, '_')
             return input
-        
+
         eltuple = namedtuple('ElTuple', 'tag, occurrence')
-        
+
         def get_sub_elements(content, multiple=False, first=None):
             sub_elements = []  #The final list of tuples to be returned
 
@@ -144,7 +148,7 @@ its DTD.'.format(xml_file))
                                 sub_elements.append(eltuple(branch.name, 'singular'))
 
             return sub_elements
-        
+
         def recursive_element_packing(element):
             if element is None:
                 return None
@@ -232,7 +236,7 @@ its DTD.'.format(xml_file))
         """
         This function will attempt to identify the publisher of the article.
         """
-        publisher_dois = {'10.1371': 'PLoS'}
+        publisher_dois = {'10.1371': 'PLoS', '10.3389': 'Frontiers'}
         #Try to look up the publisher by DOI
         if self.doi:
             try:
@@ -244,13 +248,15 @@ its DTD.'.format(xml_file))
         #If that fails, attempt to extract the publisher through inspection
         if self.dtd_name == 'JPTS':
             publisher_meta = self.metadata.front.journal_meta.publisher
-            if publisher_meta:  #Optional element
+            if publisher_meta is not None:  #Optional element
                 print(publisher_meta.publisher_name.text, type(publisher_meta.publisher_name.text))
                 if publisher_meta.publisher_name.text == 'Public Library of Science':
                     return 'PLoS'
+                elif publisher_meta.publisher_name.text == 'Frontiers Media S.A.':
+                    return 'Frontiers'
         print('Warning! Unable to identify publisher for this article!')
         return None
-            
+
     def get_DOI(self):
         """
         This function will attempt to locate the DOI string associated with the
